from NsparkleLog.core._logger import Logger
from NsparkleLog.dependencies import threading , Lock , asyncio, multiprocessing
from NsparkleLog.utils._types import Level
from NsparkleLog.core._level import Levels
from NsparkleLog.core._handler import StreamHandler , Formatter , Handler
from NsparkleLog._env import default_format , allowed_lock_type
from NsparkleLog.utils._get_lock import get_current_lock

class LogManager:
    loggers: set[Logger] = set()
    colorMode: bool = True
    lock = get_current_lock()
    handlers: list[Handler] = []
    level: Level = Levels.ON # type: ignore
    formatter: Formatter = Formatter(colorMode=True, fmt=default_format)

    @classmethod
    def _create_logger(cls, name: str, level: Level) -> Logger:
        logger = Logger(name, level)
        handlers = cls.handlers
        for handler in handlers:
            handler.setFormatter(cls.formatter)
            logger.addHandler(handler)
        cls.loggers.add(logger)
        return logger
    
    @classmethod
    def config(cls,
        handlers: list[Handler] = [StreamHandler()],
        level: Level = Levels.ON,
        colorMode: bool = True,
        formatter: Formatter = Formatter(colorMode=True, fmt=default_format),
    ):
        cls.handlers = handlers
        cls.level = level
        cls.formatter = formatter
        cls.colorMode = colorMode

    @classmethod
    def GetLogger(cls,
            name: str,
        ) -> Logger:  # type: ignore
        
        if isinstance(cls.lock, allowed_lock_type):
            with cls.lock: #type: ignore
                logger = next((logger for logger in cls.loggers if logger.name == name), None)
                if logger is None:
                    return cls._create_logger(name, cls.level) # type: ignore
                else:
                    return logger


import re

class _Color:
    red = 31
    green = 32
    yellow = 33
    blue = 34
    magenta = 35
    cyan = 36
    write = 37
    grey = 90
    background_red = 41

class Color:
    BaseStr = "\033[%d;%dm%s\033[0m"

    @classmethod
    def GetAvaliableColor(cls) -> list:
        return [i for i in _Color.__dict__ if not i.startswith("__")]

    @classmethod
    def regColor(cls,colorCode: int,colorName: str) -> None:
        setattr(_Color,colorName,colorCode)

    @classmethod
    def render(cls,string:str,color:str) -> str:
        if color.startswith("bd_"):
           color = color[3:]
           return cls.BaseStr % (1,_Color.__dict__[color],string)
        else:
            return cls.BaseStr % (0, _Color.__dict__[color], string)
        
    @classmethod
    def renderByHtml(cls, string: str) -> str:
        """
        一个接口让你允许用HTML标签来渲染颜色
        """
        pattern = re.compile(r"<(\w+)>(.*?)</\1>")
        while True:
            match = pattern.search(string)
            if not match:
                break
            color_tag = match.group(1)
            inner_text = match.group(2)
            ansi_code = cls.render(inner_text, color_tag)
            string = string[:match.start()] + ansi_code + string[match.end():]
        return string
    

if __name__ == '__main__':
    test = Color.renderByHtml("<red>hello</red> <bd_blue>world</bd_blue>")
    print(test)


import warnings
from typing import Callable, Any, Union , Optional
import inspect

class Deprecated:
    """
    Decorator for deprecated functions.
    - 对弃用函数标记的装饰器

    ### three methods to how to use this decorators
    ### 此装饰器使用的三种方法
    
    - 1. 指定原因(input reason for depercated functions)

    ```
    @Deprecated("this is a test")
    def test1():
        print("test1")
    ```

    - 2. 指定移除版本号(input depercated version)

    ```
    @Deprecated("test",d_version="0.1.0",remove=True)
    def test2():
        print("test2")
    ```
    - 3. 指定新函数名(input new function name)

    ```
    @Deprecated("test", newFuncName="test4")
    def test3():
        print("test3")
    ```
    """
    def __init__(self, 
        reason: str = "no reasons",
        d_version: Optional[str] = None,
        newFuncName: Optional[str] = None,
        remove: bool = False
    ) -> None:
        self.reason = reason
        self.d_version = d_version
        self.newFuncName = newFuncName
        self.remove = remove

    def __call__(self, func: Callable[..., Any]) -> Callable[..., Any]:
        func_signature = inspect.signature(func)
        
        def wrapper(*args: Any, **kwargs: Any) -> Any:
            formatted_reason = f"Deprecated as {self.reason},"
            formatted_reason += f"Depercated function: {func.__name__}"
            if self.d_version:
                formatted_reason += f", will be removed in version {self.d_version}"
                
            if self.remove:
                warnings.warn(f"{formatted_reason}.", DeprecationWarning, stacklevel=2)
            elif self.newFuncName:
                warnings.warn(f"{formatted_reason}, use {self.newFuncName} instead.", FutureWarning, stacklevel=2)
            else:
                warnings.warn(formatted_reason, DeprecationWarning, stacklevel=2)
                
            return func(*args, **kwargs)
        
        wrapper.__signature__ = func_signature
        return wrapper

if __name__ == "__main__":
    pass


from NsparkleLog.utils._types import Level , AnyStr
from typing import Optional
from time import struct_time
# 为了简化代码的实现,我打算用一个类来表示日志记录

class LogRecord:
    def __init__(self,
        name: str,
        threadName: Optional[str],
        threadId: Optional[int],
        filename: Optional[str],
        lineno: int,
        pathname: Optional[str],
        funcName: Optional[str],
        moduleName: Optional[str],
        ProcessId: Optional[int],
        ProcessName: Optional[str],
        message: Optional[AnyStr],
        level: Level,
        msecs: int,
        utcmsecs: float,
        timestamp: struct_time,
        utctime: struct_time
    ) -> None:
        self.name: str = name
        self.threadName = threadName
        self.threadId: Optional[int] = threadId
        self.filename = filename
        self.lineno: int = lineno
        self.pathname = pathname
        self.funcName = funcName
        self.moduleName = moduleName
        self.ProcessId: Optional[int] = ProcessId
        self.ProcessName = ProcessName
        self.message = message
        self.level: Level = level
        self.msecs = msecs
        self.utcmsecs: Optional[float] = utcmsecs
        self.timestamp: struct_time = timestamp
        self.utctime: struct_time = utctime

    def __str__(self) -> str:
        return f"""
        Log Record:
        Time: {self.timestamp}.{self.msecs}
        UTC Time: {self.utctime}.{self.utcmsecs}
        Name: {self.name}
        Thread Name: {self.threadName}
        Thread ID: {self.threadId}
        Filename: {self.filename}
        Line Number: {self.lineno}
        Pathname: {self.pathname}
        Function Name: {self.funcName}
        Module Name: {self.moduleName}
        Process ID: {self.ProcessId}
        Process Name: {self.ProcessName}
        Message: {self.message}
        Level: {self.level}
        """


from NsparkleLog.dependencies import threading , asyncio , multiprocessing , Union , Lock

def get_current_lock() -> Union[threading.Lock, asyncio.Lock, Lock]:
    """
    根据当前环境返回适当的锁类型
    """
    # 检测异步环境
    if asyncio.get_event_loop().is_running():
        return asyncio.Lock()
    # 检测多进程环境
    elif (multiprocessing.current_process().name != 'MainProcess' and 
          len(multiprocessing.active_children()) > 1):
        return multiprocessing.Lock()
    # 默认返回线程锁
    else:
        return threading.Lock()



"""
NsparkleLog 包的所有可用组件
"""

from NsparkleLog.core._manager import *
from NsparkleLog.core._level import *
from NsparkleLog.egg import *
from NsparkleLog.core._formatter import *
from NsparkleLog.core._handler import *
from NsparkleLog.core._record import *
from NsparkleLog.core._level import *
from NsparkleLog.core._excformat import *
from NsparkleLog.core._logger import *


import threading , multiprocessing
from NsparkleLog.core._level import _levelToName
from NsparkleLog.utils._types import Level

class LevelColor:
    TRACE = "bd_grey"
    DEBUG = "bd_blue"
    INFO = "bd_cyan"
    WARNING = "bd_yellow"
    WARN = "bd_yellow"
    ERROR = "bd_red"
    FATAL = "bd_background_red"

    @classmethod
    def getlevelColor(cls,level: Level) -> str:
        return getattr(cls,_levelToName[level]) # type: ignore
    
    @classmethod
    def mapLevelToColor(cls,level: Level,colorName: str) -> None:
        """ 映射日志等级到颜色 """
        setattr(cls,_levelToName[level],colorName)
    

default_format = "{levelcolor}{localtime}.{msec:03d} | {level:<6} | {ProcessName}.{threadName} | {name}.{funcName} | {filename}:{lineno} - {message}{levelColorEnd}"

allowed_lock_type = type(threading.Lock()) , type(multiprocessing.Lock())




"""
日志级别定义
"""

class Levels:
    ON = float("-inf")
    TRACE = 1
    DEBUG = 2
    INFO = 3
    WARNING = 4
    WARN = WARNING
    ERROR = 5
    FATAL = 6
    CRITICAL = FATAL
    OFF = float("inf")

_levelToName = {
    Levels.TRACE: "TRACE",
    Levels.DEBUG: "DEBUG",
    Levels.INFO: "INFO",
    Levels.WARNING: "WARN",
    Levels.ERROR: "ERROR",
    Levels.FATAL: "FATAL",
    Levels.ON: "ON",
    Levels.OFF: "OFF",
}

_nameToLevel = {v: k for k, v in _levelToName.items()}


from NsparkleLog.utils._types import Level , AnyStr
from NsparkleLog.core._level import Levels, _nameToLevel , _levelToName
from NsparkleLog.core._handler import Handler
from NsparkleLog.dependencies import threading , stderr , inspect , os , traceback , multiprocessing , functools , datetime as dt , time
from NsparkleLog.utils._color import _Color
from typing import Optional
from NsparkleLog.core._formatter import Formatter
from NsparkleLog.plugins.helpers import Deprecated
from NsparkleLog.core._handler import StreamHandler
from NsparkleLog.core._excformat import ExtractException , GetStackTrace
from NsparkleLog.core._record import LogRecord
from time import struct_time

class Logger:
    def __init__(self,
    name: str = "main",
    level = Levels.ON,
    ) -> None:
        self.name = name
        self.level = level
        self.avaliable_lvl = _nameToLevel
        self.handlers: set[Handler] = set()
        self.lock = threading.Lock()

    def addHandler(self, handler: Handler) -> None:
        """
        添加日志处理器
        """
        with self.lock:
           self.handlers.add(handler)

    def __getLogMsg(self) -> tuple[str, Optional[int], str, int, str, str, str, Optional[int], str, int, float, struct_time , struct_time]:
        threadName: str = threading.current_thread().name
        threadId: Optional[int]= threading.current_thread().ident
        frame = inspect.currentframe().f_back.f_back.f_back  # type: ignore
        filename: str = os.path.relpath(frame.f_code.co_filename, start=os.getcwd()) # type: ignore
        pathname: str = frame.f_code.co_filename # type: ignore
        lineno: int = frame.f_lineno # type: ignore
        moduleName: str = inspect.getmodule(frame).__name__ # type: ignore
        funcName: str = frame.f_code.co_name # type: ignore
        ProcessID: Optional[int] = os.getpid()
        ProcessName: str = multiprocessing.current_process().name
        msecs : int = dt.now().microsecond // 1000 
        utcmsecs: float = time.time() * 1000
        timestamp: struct_time = dt.now().timetuple()
        utctime: struct_time = time.gmtime()
        return threadName , threadId , filename , lineno , pathname , funcName , moduleName , ProcessID , ProcessName , msecs , utcmsecs , timestamp , utctime # type: ignore
    
    def catch(self,func):
        """
        实现类似loguru的catch装饰器
        """
        @functools.wraps(func)
        def wrapper(*args,**kwargs):
            try:
                return func(*args,**kwargs)
            except Exception as e:
                self.exception(e)
        return wrapper

    def _log(self, level: Level, message: AnyStr, **kwargs) -> None:
        if not self.handlers:
            raise Exception("No handlers found")
        try:
            if level >= self.level:
                threadName , threadId , filename , lineno , pathname , funcName , moduleName , ProcessId , ProcessName , msecs , utcmsecs , timestamp , utctime = self.__getLogMsg()
                for handler in self.handlers:
                    handler.handle(LogRecord(
                        name=self.name,
                        level=level,
                        message=message,
                        threadName=threadName,
                        threadId=threadId,
                        filename=filename,
                        lineno=lineno,
                        pathname=pathname,
                        funcName=funcName,
                        moduleName=moduleName,
                        ProcessId=ProcessId,
                        ProcessName=ProcessName,
                        msecs=msecs,
                        utcmsecs=utcmsecs,
                        timestamp=timestamp,
                        utctime=utctime
                    ), **kwargs)
        except Exception as e:
            err_msg = traceback.format_exc()
            stderr.write(f"{err_msg}\n")

    def setFormatter(self, formatter: Formatter) -> None:
        """
        设置日志格式
        """
        for handler in self.handlers:
            if isinstance(handler, StreamHandler):
                handler.setFormatter(formatter)

    def setLevel(self,level:Level):
        """
        设置日志等级
        """
        if level in _levelToName:
            with self.lock:
                if _levelToName[level] == "OFF":
                    self.level = Levels.OFF
                else:
                    if level < Levels.OFF:
                        self.level = level
                    else:
                        raise Exception(f"Level {level} should be less than OFF level")
        else:
            raise Exception(f"Level {level} not found")

    def addNewLevel(self, name: str, level: Level) -> None:
        """
        添加自定义等级。

        Parameters:
        - name (str): 自定义等级的名称。
        - level (Level): 自定义等级的级别。

        Raises:
        - Exception: 如果给定的等级名称已经存在，则会引发异常。

        Returns:
        - None

        Description:
        此方法允许用户添加自定义的日志等级，以便更好地适应特定需求。用户可以指定自定义等级的名称、级别、文本颜色代码以及颜色名称。添加自定义等级后，系统将根据用户的设置正确地显示和处理这些日志消息。

        Example:
        ```python
        logger.addNewLevel("test", 25)
        ```

        In this example, a new custom log level named "test" is added with level 25.
        """
        # 先判断是否已经有此级别
        if name in self.avaliable_lvl:
            raise Exception(f"Level {name} already exists")
        elif level in _levelToName:
            raise Exception(f"your Level value is same as my logging lib`s level: {_levelToName[level]}")
        else:
            with self.lock:
               self.avaliable_lvl[name] = level
               _levelToName[level] = name

    def log(self, level: Level, message: AnyStr, **kwargs) -> None:
        """
        记录一个级别为{level}的日志
        """
        self._log(level, message, **kwargs) #type: ignore

    def exception(self, exception: Exception, **kwargs) -> None:
        """
        记录一个异常
        """
        err_msg = ExtractException(type(exception), exception, exception.__traceback__)
        self._log(Levels.ERROR, err_msg, **kwargs) #type: ignore

    def isLevelEnabled(self, level: Level) -> bool:
        """
        检查给定的日志级别是否启用
        """
        if level >= self.level:
            return True
        return False
            
    def trace(self, message: AnyStr, withStackTrace: bool = False, **kwargs) -> None:
        """ 记录一个级别为trace的日志,并且可以选择是否记录堆栈跟踪 """
        if withStackTrace:
            message += f"\n{GetStackTrace(vokeInfo='in')}" # type: ignore
        self._log(Levels.TRACE, message, **kwargs) # type: ignore

    def debug(self, message: AnyStr, **kwargs) -> None:
        """ 记录一个级别为debug的日志 """
        self._log(Levels.DEBUG, message, **kwargs) # type: ignore

    def info(self, message: AnyStr, **kwargs) -> None:
        """ 记录一个级别为infomation的日志 """
        self._log(Levels.INFO, message, **kwargs) # type: ignore

    def warning(self, message: AnyStr, **kwargs) -> None:
        """ 记录一个级别为warning的日志 """
        self._log(Levels.WARNING, message, **kwargs) # type: ignore
    
    def warn(self,message: AnyStr, **kwargs) -> None:
        self._log(Levels.WARNING, message, **kwargs) # type: ignore

    def error(self, message: AnyStr, **kwargs) -> None:
        """ 记录一个级别为error的日志 """
        self._log(Levels.ERROR, message, **kwargs) # type: ignore

    def critical(self, message: AnyStr, **kwargs) -> None:
        self._log(Levels.FATAL, message, **kwargs) # type: ignore

    def fatal(self, message: AnyStr, **kwargs) -> None:
        """ 记录一个级别为fatal的日志 """
        self._log(Levels.FATAL, message, **kwargs) # type: ignore


import traceback , sys , threading ,multiprocessing
from typing import Optional

def format_stack_trace(exctype, value, tb, nested: bool = False,stackTraceMode: bool = False) -> str:
    tb_list = traceback.extract_tb(tb)

    if nested:
        exception_info = f"{exctype.__name__}: {value}\n"
    else:
        if not stackTraceMode:
            exception_info = f"Exception in process: {multiprocessing.current_process().name}, thread: {threading.current_thread().name}; {exctype.__name__}: {value}\n"
            exception_info += "Traceback (most recent call last):\n"
        else:
            exception_info = f"This is just a stack trace\n"
            exception_info += "Traceback (most recent call last):\n"

    for filename, lineno, funcname, line in tb_list:
        exception_info += f"  at {funcname} in ({filename}:{lineno})\n"
    
    # 检查是否有原因和其他信息
    cause = getattr(value, '__cause__', None)
    context = getattr(value, '__context__', None)
    
    if cause:
        exception_info += "Caused by: "
        exception_info += format_stack_trace(type(cause), cause, cause.__traceback__, nested=True)
    if context and not cause:
        exception_info += "original exception: "
        exception_info += format_stack_trace(type(context), context, context.__traceback__, nested=True)
    
    return exception_info

def ExtractException(exctype, value, tb) -> Optional[str]:
    # 获取回溯信息并格式化为字符串
    tb_str = format_stack_trace(exctype, value, tb)
    
    # 记录异常信息到日志
    exception_info = "发生异常:\n"
    exception_info += tb_str
    return exception_info

def GetStackTrace(vokeInfo: str = "in") -> str:
    """
    获取堆栈跟踪信息
    参数:
        vokeInfo: str
            指定获取堆栈跟踪的调用信息
            'in': 获取当前调用栈信息的前两层
            'out': 获取当前调用栈信息的前一层
    """
    try:
        raise Exception("获取堆栈跟踪")
    except Exception:
        # 获取当前调用栈信息的前两层
        stack = traceback.extract_stack(limit=3)
        stack_trace = "Stack Trace:\n"
        if vokeInfo == "in":
            for frame in stack[:-2]:
                func_name = frame.name
                file_name = frame.filename
                line_no = frame.lineno
                stack_trace += f"   at {func_name} in ({file_name}:{line_no})\n"
        elif vokeInfo == "out":
            for frame in stack[:-1]:
                func_name = frame.name
                file_name = frame.filename
                line_no = frame.lineno
                stack_trace += f"   at {func_name} in ({file_name}:{line_no})\n"
        else:
            raise ValueError("参数vokeInfo只能为'in'或'out'")
        return stack_trace




"""
依赖文件
"""
from typing import TypeVar, NewType , Union , Any , Literal
from sys import stderr , stdin , stdout
import multiprocessing , threading
import asyncio , re
from abc import ABC , abstractmethod
from datetime import datetime
from time import localtime
import time
import inspect , os , queue , atexit , traceback
from multiprocessing.synchronize import Lock
import multiprocessing
import functools , logging


from NsparkleLog.dependencies import stdout, stderr , threading , queue ,os , logging , time
from NsparkleLog.utils._types import Stream, AnyStr, Level
from NsparkleLog.core._level import Levels, _levelToName
from NsparkleLog.core._formatter import Formatter
from NsparkleLog.utils._get_lock import get_current_lock
from NsparkleLog._env import allowed_lock_type
from NsparkleLog.core._record import LogRecord

def isMainThreadAlive():
    return threading.main_thread().is_alive()

class Writer:
    """
    日志写入器
    """

    lock = get_current_lock()

    @classmethod
    def write(cls,stream: Stream, error: bool = False) -> None:
        if isinstance(cls.lock, allowed_lock_type):
            if error:
                with cls.lock:
                    stderr.write(f"{stream}\n")
                    stderr.flush()
            else:
                with cls.lock:
                    stdout.write(f"{stream}\n")
                    stdout.flush()


class Handler:
    def __init__(self) -> None:
        self.formatter = None
        self.level = Levels.ON
        self.name = None
        self.log_msg: str = ""
        self.getLevel = _levelToName

    def setLevel(self, level: Level) -> None:
        """ 设置等级 """
        self.level = level

    def setFormatter(self, formatter: Formatter) -> None:
        """ 设置格式 """
        self.formatter = formatter

    def handle(
        self,
        record: LogRecord
    ) -> str:
        """ 主要的处理程序 """
        
        if self.formatter is None:
            self.formatter = Formatter()

        formatted_msg = self.formatter.format(record)
        
        return formatted_msg

class StreamHandler(Handler, Writer):
    def __init__(self, error: bool = False) -> None:
        super().__init__()
        
    def handle(self, record: LogRecord) -> None:
        string =  super().handle(record)
        if record.level >= self.level:
           if record.level >= Levels.ERROR:
               self.write(string, error=True)
           else: 
               self.write(string, error=False)
        return

class FileHandler(Handler):
    """
    日志文件写入器
    """
    def __init__(self, filename: str, mode: str = "a+", encoding: str = "utf-8") -> None:
        super().__init__()
        self.filename = filename
        self.mode = mode
        self.encoding = encoding
        self.lock = get_current_lock()
        self.queue = queue.Queue()
        if self.formatter is not None:
            self.formatter.colorMode = False
        self.writeThread = threading.Thread(target=self.writeToFile)
        self.writeThread.start()

    def handle(self, record: LogRecord) -> None:
        string =  super().handle(record)
        if record.level >= self.level:
           self.queue.put_nowait(string)
        return

    def writeToFile(self) -> None:
        if isinstance(self.lock, allowed_lock_type):
            with open(self.filename, self.mode, encoding=self.encoding) as f:
                while True:
                    try:
                        string = self.queue.get(timeout=1.0)
                        with self.lock:
                            f.write(f"{string}\n")
                            f.flush()
                        self.queue.task_done()
                    except queue.Empty:
                        if  not isMainThreadAlive():
                            break
                        else:
                            continue
                    except Exception as e:
                        stderr.write(f"{e}\n")
        else:
           raise RuntimeError("lock is not a threading.Lock or mulitprocessing.Lock")
        
class QueueHandler(Handler):
    """
    日志队列写入器
    """
    def __init__(self) -> None:
        super().__init__()
        self._queue = queue.Queue()

    def handle(self, record: LogRecord) -> None:
        string =  super().handle(record)
        if record.level >= self.level:
            self._queue.put_nowait(string)
        return
    
    def out(self,timeout:float = 1.0) -> str:
        return self._queue.get(timeout=timeout)
    
class NullHandler(Handler):
    """
    空日志处理器
    """
    def __init__(self) -> None:
        super().__init__()

    def handle(self, record: LogRecord) -> None:
        pass

class NsparkleLogHandler(logging.Handler, Handler , Writer):
    """
    转发logging日志消息到 NsparkleLog
    """
    def __init__(self) -> None:
        logging.Handler.__init__(self)
        Handler.__init__(self)
        Writer.__init__(self)
        self.baseHandler = Handler()
        self.logging_to_NsparkleLogLvl = {
            logging.CRITICAL: Levels.FATAL,
            logging.ERROR: Levels.ERROR,
            logging.WARNING: Levels.WARN,
            logging.INFO: Levels.INFO,
            logging.DEBUG: Levels.DEBUG,
            logging.NOTSET: Levels.OFF
        }
        self.colorLevel: dict[int, str] = { # type: ignore
            0 : "bd_grey",
            10 : "bd_blue",
            20 : "bd_cyan",
            30 : "bd_yellow",
            40 : "bd_red",
            50 : "bd_background_red",
    }

    def emit(self, record: logging.LogRecord) -> None:
        new_record: LogRecord = LogRecord(
            name=record.name,
            level=self.logging_to_NsparkleLogLvl[record.levelno],
            pathname=record.pathname,
            lineno=record.lineno,
            threadName=record.threadName,
            threadId=record.thread,
            message=record.getMessage(),
            timestamp=time.localtime(record.created),
            ProcessName=record.processName,
            ProcessId=record.process,
            funcName=record.funcName,
            filename=record.filename,
            moduleName=record.module,
            msecs=int(record.msecs),
            utcmsecs=int(record.msecs),
            utctime=time.gmtime(record.created)
        )
        if self.baseHandler.formatter is not None:
            self.baseHandler.formatter.colorMode = True
        string = self.baseHandler.handle(new_record)
        if record.levelno >= self.level:
            if record.levelno >= logging.ERROR:
                self.write(string, error=True)
            else:
                self.write(string, error=False)


class RotatingFileHandler(Handler):
    """
    日志文件按大小轮换写入器
    """
    def __init__(self) -> None:
       raise NotImplementedError


from setuptools import setup, find_packages
from NsparkleLog.metadata import __version__ , __author__ , __packageName__
from NsparkleLog import logger

def load_requirements(filepath: str = "requirements.txt") -> list[str]:
    try:
       with open(filepath, 'r') as f:
           return [line.strip() for line in f.readlines()]
    except FileNotFoundError:
        return []
    except Exception as e:
        return []
    
# 记录更新日志
logger.info(f"版本: {__version__}")
logger.info(f"作者: {__author__}")
logger.info(f"包名: {__packageName__}")

setup(
    name=__packageName__,
    version=__version__,
    packages=find_packages(),
    author=__author__,
    author_email='3072252442@qq.com',
    description='A useful logging library for Python',
    long_description=open('Readme.md','r',encoding='utf-8').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/KOKOMI12345/NewSparkleLogging',
    requires=load_requirements(),  # 依赖
    license='MIT',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
)



from NsparkleLog.utils._types import AnyStr, Level
from NsparkleLog.dependencies import time
from NsparkleLog.core._level import _levelToName
from NsparkleLog.utils._color import Color
from NsparkleLog._env import default_format , LevelColor
from NsparkleLog.core._record import LogRecord

class Formatter:
    """
    同时你还可以用颜色标签来标记日志的颜色
    ### 支持的格式占位符: 
     - {name}: 日志记录器名字
     - {threadName}: 线程名字
     - {threadId}: 线程ID
     - {filename}: 文件名
     - {pathname}: 文件路径
     - {lineno}: 行号
     - {funcName}: 函数名
     - {moduleName}: 模块名
     - {ProcessId}: 进程ID
     - {ProcessName}: 进程名
     - {message}: 消息
     - {level}: 日志级别
     - {localtime}: 本地时间
     - {msecs}: 本地时间毫秒
     - {utcmsecs}: UTC时间毫秒
     - {utctime}: UTC时间
     - {timestamp}: 时间戳(本地时间)
     - {levelcolor}: 日志颜色(随等级变化)
     - {levelColorEnd}: 结束颜色(颜色结束标签)
    """
    def __init__(self,
        colorMode:bool = False,
        fmt: str = default_format,
        datefmt: str = "%Y-%m-%d %H:%M:%S"
        ) -> None:
        self._fmt = fmt
        self.colorMode = colorMode
        self._date = datefmt

    def format(self, 
        record: LogRecord,
        ) -> str:
        formatted_msg = self._fmt.format(
            timestamp=time.strftime(self._date,record.timestamp),
            msec=record.msecs,
            utcmsec=record.utcmsecs,
            localtime = time.strftime(self._date,record.timestamp),
            utctime=time.strftime(self._date,record.utctime),
            level=_levelToName[record.level],
            threadName=record.threadName,
            threadId=record.threadId,
            filename=record.filename,
            pathname=record.pathname,
            ProcessId=record.ProcessId,
            ProcessName=record.ProcessName,
            lineno=record.lineno,
            funcName=record.funcName,
            moduleName=record.moduleName,
            name=record.name,
            message=record.message,
            levelcolor=f"<{LevelColor.getlevelColor(record.level)}>", # type: ignore
            levelColorEnd=f"</{LevelColor.getlevelColor(record.level)}>" # type: ignore
        )
        if self.colorMode:
            return Color.renderByHtml(formatted_msg)
        return formatted_msg